@@ -14,7 +14,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import print_function
+from __future__ import print_function, unicode_literals
 
 import logging
 import signal
@@ -23,6 +23,7 @@ import socket
 from optparse import OptionParser, OptionGroup, SUPPRESS_HELP
 from os import environ, walk, _exit as os_exit
 from os.path import isfile, isdir, join
+from six import text_type
 from sys import argv as sys_argv, exit, stderr
 from time import gmtime, strftime
 
@@ -31,14 +32,19 @@ from swiftclient.utils import config_true_value, generate_temp_url, prt_bytes
 from swiftclient.multithreading import OutputManager
 from swiftclient.exceptions import ClientException
 from swiftclient import __version__ as client_version
-from swiftclient.service import SwiftService, SwiftError, SwiftUploadObject
+from swiftclient.service import SwiftService, SwiftError, \
+    SwiftUploadObject, get_conn
 from swiftclient.command_helpers import print_account_stats, \
     print_container_stats, print_object_stats
 
+try:
+    from shlex import quote as sh_quote
+except ImportError:
+    from pipes import quote as sh_quote
 
 BASENAME = 'swift'
-commands = ('delete', 'download', 'list', 'post',
-            'stat', 'upload', 'capabilities', 'info', 'tempurl')
+commands = ('delete', 'download', 'list', 'post', 'stat', 'upload',
+            'capabilities', 'info', 'tempurl', 'auth')
 
 
 def immediate_exit(signum, frame):
@@ -106,7 +112,7 @@ def st_delete(parser, args, output_manager):
                 if '/' in container:
                     output_manager.error(
                         'WARNING: / in container name; you '
-                        'might have meant %r instead of %r.' % (
+                        "might have meant '%s' instead of '%s'." % (
                         container.replace('/', ' ', 1), container)
                     )
                     return
@@ -146,9 +152,11 @@ def st_delete(parser, args, output_manager):
 
 
 st_download_options = '''[--all] [--marker] [--prefix <prefix>]
-                      [--output <out_file>] [--object-threads <threads>]
+                      [--output <out_file>] [--output-dir <out_directory>]
+                      [--object-threads <threads>]
                       [--container-threads <threads>] [--no-download]
-                      [--skip-identical] <container> <object>
+                      [--skip-identical] [--remove-prefix]
+                      <container> <object>
 '''
 
 st_download_help = '''
@@ -167,9 +175,15 @@ Optional arguments:
   --marker              Marker to use when starting a container or account
                         download.
   --prefix <prefix>     Only download items beginning with <prefix>
+  --remove-prefix       An optional flag for --prefix <prefix>, use this
+                        option to download items without <prefix>
   --output <out_file>   For a single file download, stream the output to
                         <out_file>. Specifying "-" as <out_file> will
                         redirect to stdout.
+  --output-dir <out_directory>
+                        An optional directory to which to store objects.
+                        By default, all objects are recreated in the current
+                        directory.
   --object-threads <threads>
                         Number of threads to use for downloading objects.
                         Default is 10.
@@ -203,6 +217,14 @@ def st_download(parser, args, output_manager):
         '-o', '--output', dest='out_file', help='For a single '
         'download, stream the output to <out_file>. '
         'Specifying "-" as <out_file> will redirect to stdout.')
+    parser.add_option(
+        '-D', '--output-dir', dest='out_directory',
+        help='An optional directory to which to store objects. '
+        'By default, all objects are recreated in the current directory.')
+    parser.add_option(
+        '-r', '--remove-prefix', action='store_true', dest='remove_prefix',
+        default=False, help='An optional flag for --prefix <prefix>, '
+        'use this option to download items without <prefix>.')
     parser.add_option(
         '', '--object-threads', type=int,
         default=10, help='Number of threads to use for downloading objects. '
@@ -233,6 +255,12 @@ def st_download(parser, args, output_manager):
     if options.out_file and len(args) != 2:
         exit('-o option only allowed for single file downloads')
 
+    if not options.prefix:
+        options.remove_prefix = False
+
+    if options.out_directory and len(args) == 2:
+        exit('Please use -o option for single file downloads and renames')
+
     if (not args and not options.yes_all) or (args and options.yes_all):
         output_manager.error('Usage: %s download %s\n%s', BASENAME,
                              st_download_options, st_download_help)
@@ -249,7 +277,7 @@ def st_download(parser, args, output_manager):
                 if '/' in container:
                     output_manager.error(
                         'WARNING: / in container name; you '
-                        'might have meant %r instead of %r.' % (
+                        "might have meant '%s' instead of '%s'." % (
                         container.replace('/', ' ', 1), container)
                     )
                     return
@@ -500,7 +528,7 @@ def st_stat(parser, args, output_manager):
                 if '/' in container:
                     output_manager.error(
                         'WARNING: / in container name; you might have '
-                        'meant %r instead of %r.' %
+                        "meant '%s' instead of '%s'." %
                         (container.replace('/', ' ', 1), container))
                     return
                 args = args[1:]
@@ -604,7 +632,7 @@ def st_post(parser, args, output_manager):
                 if '/' in container:
                     output_manager.error(
                         'WARNING: / in container name; you might have '
-                        'meant %r instead of %r.' %
+                        "meant '%s' instead of '%s'." %
                         (args[0].replace('/', ' ', 1), args[0]))
                     return
                 args = args[1:]
@@ -763,6 +791,9 @@ def st_upload(parser, args, output_manager):
                 return
 
             options.segment_size = str((1024 ** size_mod) * multiplier)
+        if int(options.segment_size) <= 0:
+            output_manager.error("segment-size should be positive")
+            return
 
     _opts = vars(options)
     _opts['object_uu_threads'] = options.object_threads
@@ -841,7 +872,7 @@ def st_upload(parser, args, output_manager):
                             msg = ': %s' % error
                         output_manager.warning(
                             'Warning: failed to create container '
-                            '%r%s', container, msg
+                            "'%s'%s", container, msg
                         )
                     else:
                         output_manager.error("%s" % error)
@@ -905,6 +936,46 @@ def st_capabilities(parser, args, output_manager):
 
 st_info = st_capabilities
 
+st_auth_help = '''
+Display auth related authentication variables in shell friendly format.
+
+  Commands to run to export storage url and auth token into
+  OS_STORAGE_URL and OS_AUTH_TOKEN:
+
+      swift auth
+
+  Commands to append to a runcom file (e.g. ~/.bashrc, /etc/profile) for
+  automatic authentication:
+
+      swift auth -v -U test:tester -K testing \
+          -A http://localhost:8080/auth/v1.0
+
+'''.strip('\n')
+
+
+def st_auth(parser, args, thread_manager):
+    (options, args) = parse_args(parser, args)
+    _opts = vars(options)
+    if options.verbose > 1:
+        if options.auth_version in ('1', '1.0'):
+            print('export ST_AUTH=%s' % sh_quote(options.auth))
+            print('export ST_USER=%s' % sh_quote(options.user))
+            print('export ST_KEY=%s' % sh_quote(options.key))
+        else:
+            print('export OS_IDENTITY_API_VERSION=%s' % sh_quote(
+                options.auth_version))
+            print('export OS_AUTH_VERSION=%s' % sh_quote(options.auth_version))
+            print('export OS_AUTH_URL=%s' % sh_quote(options.auth))
+            for k, v in sorted(_opts.items()):
+                if v and k.startswith('os_') and \
+                        k not in ('os_auth_url', 'os_options'):
+                    print('export %s=%s' % (k.upper(), sh_quote(v)))
+    else:
+        conn = get_conn(_opts)
+        url, token = conn.get_auth()
+        print('export OS_STORAGE_URL=%s' % sh_quote(url))
+        print('export OS_AUTH_TOKEN=%s' % sh_quote(token))
+
 
 st_tempurl_options = '<method> <seconds> <path> <key>'
 
@@ -912,14 +983,14 @@ st_tempurl_options = '<method> <seconds> <path> <key>'
 st_tempurl_help = '''
 Generates a temporary URL for a Swift object.
 
-Positions arguments:
-  [method]              An HTTP method to allow for this temporary URL.
+Positional arguments:
+  <method>              An HTTP method to allow for this temporary URL.
                         Usually 'GET' or 'PUT'.
-  [seconds]             The amount of time in seconds the temporary URL will
+  <seconds>             The amount of time in seconds the temporary URL will
                         be valid for.
-  [path]                The full path to the Swift object. Example:
+  <path>                The full path to the Swift object. Example:
                         /v1/AUTH_account/c/o.
-  [key]                 The secret temporary URL key set on the Swift cluster.
+  <key>                 The secret temporary URL key set on the Swift cluster.
                         To set a key, run \'swift post -m
                         "Temp-URL-Key:b3968d0207b54ece87cccc06515a89d4"\'
 '''.strip('\n')
@@ -1034,8 +1105,10 @@ def main(arguments=None):
     else:
         argv = sys_argv
 
+    argv = [a if isinstance(a, text_type) else a.decode('utf-8') for a in argv]
+
     version = client_version
-    parser = OptionParser(version='%%prog %s' % version,
+    parser = OptionParser(version='python-swiftclient %s' % version,
                           usage='''
 usage: %%prog [--version] [--help] [--os-help] [--snet] [--verbose]
              [--debug] [--info] [--quiet] [--auth <auth_url>]
@@ -1073,7 +1146,8 @@ Positional arguments:
                          or object.
     upload               Uploads files or directories to the given container.
     capabilities         List cluster capabilities.
-    tempurl              Create a temporary URL
+    tempurl              Create a temporary URL.
+    auth                 Display auth related environment variables.
 
 Examples:
   %%prog download --help
