@@ -14,7 +14,6 @@
 # limitations under the License.
 
 import logging
-import json
 
 try:
     from unittest import mock
@@ -28,9 +27,9 @@ import warnings
 import tempfile
 from hashlib import md5
 from six.moves.urllib.parse import urlparse
-from six.moves import reload_module
 
-from .utils import MockHttpTest, fake_get_auth_keystone, StubResponse
+from .utils import (MockHttpTest, fake_get_auth_keystone, StubResponse,
+                    FakeKeystone, _make_fake_import_keystone_client)
 
 from swiftclient.utils import EMPTY_ETAG
 from swiftclient import client as c
@@ -64,36 +63,6 @@ class TestClientException(testtools.TestCase):
             self.assertTrue(value in str(exc))
 
 
-class TestJsonImport(testtools.TestCase):
-
-    def tearDown(self):
-        reload_module(json)
-
-        try:
-            import simplejson
-        except ImportError:
-            pass
-        else:
-            reload_module(simplejson)
-        super(TestJsonImport, self).tearDown()
-
-    def test_any(self):
-        self.assertTrue(hasattr(c, 'json_loads'))
-
-    def test_no_simplejson_falls_back_to_stdlib_when_reloaded(self):
-        # break simplejson
-        try:
-            import simplejson
-        except ImportError:
-            # not installed, so we don't have to break it for these tests
-            pass
-        else:
-            delattr(simplejson, 'loads')  # break simple json
-            reload_module(c)  # reload to repopulate json_loads
-
-        self.assertEqual(c.json_loads, json.loads)
-
-
 class MockHttpResponse(object):
     def __init__(self, status=0, headers=None, verify=False):
         self.status = status
@@ -106,20 +75,31 @@ class MockHttpResponse(object):
         self.headers = {'etag': '"%s"' % EMPTY_ETAG}
         if headers:
             self.headers.update(headers)
+        self.closed = False
 
         class Raw(object):
-            def read(self):
-                pass
-        self.raw = Raw()
+            def __init__(self, headers):
+                self.headers = headers
+
+            def read(self, **kw):
+                return ""
+
+            def getheader(self, name, default):
+                return self.headers.get(name, default)
+
+        self.raw = Raw(headers)
 
     def read(self):
         return ""
 
+    def close(self):
+        self.closed = True
+
     def getheader(self, name, default):
         return self.headers.get(name, default)
 
     def getheaders(self):
-        return {"key1": "value1", "key2": "value2"}
+        return dict(self.headers)
 
     def fake_response(self):
         return self
@@ -221,8 +201,6 @@ class TestHttpHelpers(MockHttpTest):
         ua = req_headers.get('user-agent', 'XXX-MISSING-XXX')
         self.assertEqual(ua, 'a-new-default')
 
-# TODO: following tests are placeholders, need more tests, better coverage
-
 
 class TestGetAuth(MockHttpTest):
 
@@ -260,6 +238,59 @@ class TestGetAuth(MockHttpTest):
         # the full plumbing into the requests's 'verify' option
         self.assertIn('invalid_certificate', str(e))
 
+    def test_auth_v1_timeout(self):
+        # this test has some overlap with
+        # TestConnection.test_timeout_passed_down but is required to check that
+        # get_auth does the right thing when it is not passed a timeout arg
+        orig_http_connection = c.http_connection
+        timeouts = []
+
+        def fake_request_handler(*a, **kw):
+            if 'timeout' in kw:
+                timeouts.append(kw['timeout'])
+            else:
+                timeouts.append(None)
+            return MockHttpResponse(
+                status=200,
+                headers={
+                    'x-auth-token': 'a_token',
+                    'x-storage-url': 'http://files.example.com/v1/AUTH_user'})
+
+        def fake_connection(*a, **kw):
+            url, conn = orig_http_connection(*a, **kw)
+            conn._request = fake_request_handler
+            return url, conn
+
+        with mock.patch('swiftclient.client.http_connection', fake_connection):
+            c.get_auth('http://www.test.com', 'asdf', 'asdf',
+                       auth_version="1.0", timeout=42.0)
+            c.get_auth('http://www.test.com', 'asdf', 'asdf',
+                       auth_version="1.0", timeout=None)
+            c.get_auth('http://www.test.com', 'asdf', 'asdf',
+                       auth_version="1.0")
+
+        self.assertEqual(timeouts, [42.0, None, None])
+
+    def test_auth_v2_timeout(self):
+        # this test has some overlap with
+        # TestConnection.test_timeout_passed_down but is required to check that
+        # get_auth does the right thing when it is not passed a timeout arg
+        fake_ks = FakeKeystone(endpoint='http://some_url', token='secret')
+        with mock.patch('swiftclient.client._import_keystone_client',
+                        _make_fake_import_keystone_client(fake_ks)):
+            c.get_auth('http://www.test.com', 'asdf', 'asdf',
+                       os_options=dict(tenant_name='tenant'),
+                       auth_version="2.0", timeout=42.0)
+            c.get_auth('http://www.test.com', 'asdf', 'asdf',
+                       os_options=dict(tenant_name='tenant'),
+                       auth_version="2.0", timeout=None)
+            c.get_auth('http://www.test.com', 'asdf', 'asdf',
+                       os_options=dict(tenant_name='tenant'),
+                       auth_version="2.0")
+        self.assertEqual(3, len(fake_ks.calls))
+        timeouts = [call['timeout'] for call in fake_ks.calls]
+        self.assertEqual([42.0, None, None], timeouts)
+
     def test_auth_v2_with_tenant_name(self):
         os_options = {'tenant_name': 'asdf'}
         req_args = {'auth_version': '2.0'}
@@ -655,6 +686,41 @@ class TestGetObject(MockHttpTest):
             }),
         ])
 
+    def test_chunk_size_read_method(self):
+        conn = c.Connection('http://auth.url/', 'some_user', 'some_key')
+        with mock.patch('swiftclient.client.get_auth_1_0') as mock_get_auth:
+            mock_get_auth.return_value = ('http://auth.url/', 'tToken')
+            c.http_connection = self.fake_http_connection(200, body='abcde')
+            __, resp = conn.get_object('asdf', 'asdf', resp_chunk_size=3)
+            self.assertTrue(hasattr(resp, 'read'))
+            self.assertEquals(resp.read(3), 'abc')
+            self.assertEquals(resp.read(None), 'de')
+            self.assertEquals(resp.read(), '')
+
+    def test_chunk_size_iter(self):
+        conn = c.Connection('http://auth.url/', 'some_user', 'some_key')
+        with mock.patch('swiftclient.client.get_auth_1_0') as mock_get_auth:
+            mock_get_auth.return_value = ('http://auth.url/', 'tToken')
+            c.http_connection = self.fake_http_connection(200, body='abcde')
+            __, resp = conn.get_object('asdf', 'asdf', resp_chunk_size=3)
+            self.assertTrue(hasattr(resp, 'next'))
+            self.assertEquals(next(resp), 'abc')
+            self.assertEquals(next(resp), 'de')
+            self.assertRaises(StopIteration, next, resp)
+
+    def test_chunk_size_read_and_iter(self):
+        conn = c.Connection('http://auth.url/', 'some_user', 'some_key')
+        with mock.patch('swiftclient.client.get_auth_1_0') as mock_get_auth:
+            mock_get_auth.return_value = ('http://auth.url/', 'tToken')
+            c.http_connection = self.fake_http_connection(200, body='abcdef')
+            __, resp = conn.get_object('asdf', 'asdf', resp_chunk_size=2)
+            self.assertTrue(hasattr(resp, 'read'))
+            self.assertEquals(resp.read(3), 'abc')
+            self.assertEquals(next(resp), 'de')
+            self.assertEquals(resp.read(), 'f')
+            self.assertRaises(StopIteration, next, resp)
+            self.assertEquals(resp.read(), '')
+
 
 class TestHeadObject(MockHttpTest):
 
@@ -925,7 +991,7 @@ class TestDeleteObject(MockHttpTest):
 class TestGetCapabilities(MockHttpTest):
 
     def test_ok(self):
-        conn = self.fake_http_connection(200, body='{}')
+        conn = self.fake_http_connection(200, body=b'{}')
         http_conn = conn('http://www.test.com/info')
         info = c.get_capabilities(http_conn)
         self.assertRequests([
@@ -946,7 +1012,7 @@ class TestGetCapabilities(MockHttpTest):
         }
         auth_v1_response = StubResponse(headers=auth_headers)
         stub_info = {'swift': {'fake': True}}
-        info_response = StubResponse(body=json.dumps(stub_info))
+        info_response = StubResponse(body=b'{"swift":{"fake":true}}')
         fake_conn = self.fake_http_connection(auth_v1_response, info_response)
 
         conn = c.Connection('http://auth.example.com/auth/v1.0',
@@ -964,7 +1030,7 @@ class TestGetCapabilities(MockHttpTest):
         fake_keystone = fake_get_auth_keystone(
             storage_url='http://storage.example.com/v1/AUTH_test')
         stub_info = {'swift': {'fake': True}}
-        info_response = StubResponse(body=json.dumps(stub_info))
+        info_response = StubResponse(body=b'{"swift":{"fake":true}}')
         fake_conn = self.fake_http_connection(info_response)
 
         os_options = {'project_id': 'test'}
@@ -982,7 +1048,7 @@ class TestGetCapabilities(MockHttpTest):
 
     def test_conn_get_capabilities_with_url_param(self):
         stub_info = {'swift': {'fake': True}}
-        info_response = StubResponse(body=json.dumps(stub_info))
+        info_response = StubResponse(body=b'{"swift":{"fake":true}}')
         fake_conn = self.fake_http_connection(info_response)
 
         conn = c.Connection('http://auth.example.com/auth/v1.0',
@@ -998,7 +1064,7 @@ class TestGetCapabilities(MockHttpTest):
 
     def test_conn_get_capabilities_with_preauthurl_param(self):
         stub_info = {'swift': {'fake': True}}
-        info_response = StubResponse(body=json.dumps(stub_info))
+        info_response = StubResponse(body=b'{"swift":{"fake":true}}')
         fake_conn = self.fake_http_connection(info_response)
 
         storage_url = 'http://storage.example.com/v1/AUTH_test'
@@ -1014,7 +1080,7 @@ class TestGetCapabilities(MockHttpTest):
 
     def test_conn_get_capabilities_with_os_options(self):
         stub_info = {'swift': {'fake': True}}
-        info_response = StubResponse(body=json.dumps(stub_info))
+        info_response = StubResponse(body=b'{"swift":{"fake":true}}')
         fake_conn = self.fake_http_connection(info_response)
 
         storage_url = 'http://storage.example.com/v1/AUTH_test'
@@ -1036,6 +1102,29 @@ class TestGetCapabilities(MockHttpTest):
 
 class TestHTTPConnection(MockHttpTest):
 
+    def test_bad_url_scheme(self):
+        url = u'www.test.com'
+        exc = self.assertRaises(c.ClientException, c.http_connection, url)
+        expected = u'Unsupported scheme "" in url "www.test.com"'
+        self.assertEqual(expected, str(exc))
+
+        url = u'://www.test.com'
+        exc = self.assertRaises(c.ClientException, c.http_connection, url)
+        expected = u'Unsupported scheme "" in url "://www.test.com"'
+        self.assertEqual(expected, str(exc))
+
+        url = u'blah://www.test.com'
+        exc = self.assertRaises(c.ClientException, c.http_connection, url)
+        expected = u'Unsupported scheme "blah" in url "blah://www.test.com"'
+        self.assertEqual(expected, str(exc))
+
+    def test_ok_url_scheme(self):
+        for scheme in ('http', 'https', 'HTTP', 'HTTPS'):
+            url = u'%s://www.test.com' % scheme
+            parsed_url, conn = c.http_connection(url)
+            self.assertEqual(scheme.lower(), parsed_url.scheme)
+            self.assertEqual(u'%s://www.test.com' % scheme, conn.url)
+
     def test_ok_proxy(self):
         conn = c.http_connection(u'http://www.test.com/',
                                  proxy='http://localhost:8080')
@@ -1057,6 +1146,17 @@ class TestHTTPConnection(MockHttpTest):
         conn = c.http_connection(u'http://www.test.com/', insecure=True)
         self.assertEqual(conn[1].requests_args['verify'], False)
 
+    def test_response_connection_released(self):
+        _parsed_url, conn = c.http_connection(u'http://www.test.com/')
+        conn.resp = MockHttpResponse()
+        conn.resp.raw = mock.Mock()
+        conn.resp.raw.read.side_effect = ["Chunk", ""]
+        resp = conn.getresponse()
+        self.assertFalse(resp.closed)
+        self.assertEqual("Chunk", resp.read())
+        self.assertFalse(resp.read())
+        self.assertTrue(resp.closed)
+
 
 class TestConnection(MockHttpTest):
 
@@ -1131,7 +1231,7 @@ class TestConnection(MockHttpTest):
 
             for method, args in method_signatures:
                 c.http_connection = self.fake_http_connection(
-                    200, body='[]', storage_url=static_url)
+                    200, body=b'[]', storage_url=static_url)
                 method(*args)
                 self.assertEqual(len(self.request_log), 1)
                 for request in self.iter_request_log():
@@ -1381,6 +1481,66 @@ class TestConnection(MockHttpTest):
             ('HEAD', '/v1/AUTH_pre_url', '', {'x-auth-token': 'post_token'}),
         ])
 
+    def test_timeout_passed_down(self):
+        # We want to avoid mocking http_connection(), and most especially
+        # avoid passing it down in argument. However, we cannot simply
+        # instantiate C=Connection(), then shim C.http_conn. Doing so would
+        # avoid some of the code under test (where _retry() invokes
+        # http_connection()), and would miss get_auth() completely.
+        # So, with regret, we do mock http_connection(), but with a very
+        # light shim that swaps out _request() as originally intended.
+
+        orig_http_connection = c.http_connection
+
+        timeouts = []
+
+        def my_request_handler(*a, **kw):
+            if 'timeout' in kw:
+                timeouts.append(kw['timeout'])
+            else:
+                timeouts.append(None)
+            return MockHttpResponse(
+                status=200,
+                headers={
+                    'x-auth-token': 'a_token',
+                    'x-storage-url': 'http://files.example.com/v1/AUTH_user'})
+
+        def shim_connection(*a, **kw):
+            url, conn = orig_http_connection(*a, **kw)
+            conn._request = my_request_handler
+            return url, conn
+
+        # v1 auth
+        conn = c.Connection(
+            'http://auth.example.com', 'user', 'password', timeout=33.0)
+        with mock.patch.multiple('swiftclient.client',
+                                 http_connection=shim_connection,
+                                 sleep=mock.DEFAULT):
+            conn.head_account()
+
+        # 1 call is through get_auth, 1 call is HEAD for account
+        self.assertEqual(timeouts, [33.0, 33.0])
+
+        # v2 auth
+        timeouts = []
+        conn = c.Connection(
+            'http://auth.example.com', 'user', 'password', timeout=33.0,
+            os_options=dict(tenant_name='tenant'), auth_version=2.0)
+        fake_ks = FakeKeystone(endpoint='http://some_url', token='secret')
+        with mock.patch('swiftclient.client._import_keystone_client',
+                        _make_fake_import_keystone_client(fake_ks)):
+            with mock.patch.multiple('swiftclient.client',
+                                     http_connection=shim_connection,
+                                     sleep=mock.DEFAULT):
+                conn.head_account()
+
+        # check timeout is passed to keystone client
+        self.assertEqual(1, len(fake_ks.calls))
+        self.assertTrue('timeout' in fake_ks.calls[0])
+        self.assertEqual(33.0, fake_ks.calls[0].get('timeout'))
+        # check timeout passed to HEAD for account
+        self.assertEqual(timeouts, [33.0])
+
     def test_reset_stream(self):
 
         class LocalContents(object):
@@ -1441,7 +1601,8 @@ class TestConnection(MockHttpTest):
                 return ''
 
         def local_http_connection(url, proxy=None, cacert=None,
-                                  insecure=False, ssl_compression=True):
+                                  insecure=False, ssl_compression=True,
+                                  timeout=None):
             parsed = urlparse(url)
             return parsed, LocalConnection()
 
